import unittest, sys
import io
import unittest.mock
import math
import numpy as np
from sklearn.datasets import make_blobs
from sklearn.preprocessing import MinMaxScaler
from sklearn.model_selection import train_test_split

from tmc import points

from tmc.utils import load, get_out

main = load('src.main', 'main')

def dist(a, b):
    sum = 0
    for i in range(len(a)):
        sum = sum + (a[i] - b[i])**2
    return math.sqrt(sum)


def main_test(X_train, X_test, Y_train, Y_test):
    Y_predict = np.empty(len(Y_test), dtype=np.int64)
    
    k = 3
    N_test = len(X_test)
    N_train = len(X_train)
    
    for i in range(N_test):
        D = np.empty(N_train)
        for j in range(N_train):
            D[j] = dist(X_train[j], X_test[i])
    
        near = np.argsort(D)[:k ]
    
        Y_predict[i] = np.round(np.mean(Y_train[near]))

    return str(Y_predict)

@points('3-6')
class MainTest(unittest.TestCase):
   
    def test_answer_one(self):
        X, Y = make_blobs(n_samples=16, n_features=2, centers=2, center_box=(-2,2))
        X = MinMaxScaler().fit(X).transform(X)
        X_train, X_test, Y_train, Y_test = train_test_split(X, Y, test_size = 2)

        main(X_train, X_test, Y_train, Y_test)
        output = get_out()
        expected = main_test(X_train, X_test, Y_train, Y_test)
        self.assertEqual(output, expected)

    def test_answer_two(self):
        X, Y = make_blobs(n_samples=16, n_features=2, centers=2, center_box=(-2,2))
        X = MinMaxScaler().fit(X).transform(X)
        X_train, X_test, Y_train, Y_test = train_test_split(X, Y, test_size = 2)

        main(X_train, X_test, Y_train, Y_test)
        output = get_out()
        expected = main_test(X_train, X_test, Y_train, Y_test)
        self.assertEqual(output, expected)

    def test_answer_three(self):
        X, Y = make_blobs(n_samples=16, n_features=2, centers=2, center_box=(-2,2))
        X = MinMaxScaler().fit(X).transform(X)
        X_train, X_test, Y_train, Y_test = train_test_split(X, Y, test_size = 2)

        main(X_train, X_test, Y_train, Y_test)
        output = get_out()
        expected = main_test(X_train, X_test, Y_train, Y_test)
        self.assertEqual(output, expected)

    def test_answer_four(self):
        X, Y = make_blobs(n_samples=16, n_features=2, centers=2, center_box=(-2,2))
        X = MinMaxScaler().fit(X).transform(X)
        X_train, X_test, Y_train, Y_test = train_test_split(X, Y, test_size = 2)

        main(X_train, X_test, Y_train, Y_test)
        output = get_out()
        expected = main_test(X_train, X_test, Y_train, Y_test)
        self.assertEqual(output, expected)

    def test_answer_five(self):
        X, Y = make_blobs(n_samples=16, n_features=2, centers=2, center_box=(-2,2))
        X = MinMaxScaler().fit(X).transform(X)
        X_train, X_test, Y_train, Y_test = train_test_split(X, Y, test_size = 2)

        main(X_train, X_test, Y_train, Y_test)
        output = get_out()
        expected = main_test(X_train, X_test, Y_train, Y_test)
        self.assertEqual(output, expected)

    def test_answer_six(self):
        X, Y = make_blobs(n_samples=16, n_features=2, centers=2, center_box=(-2,2))
        X = MinMaxScaler().fit(X).transform(X)
        X_train, X_test, Y_train, Y_test = train_test_split(X, Y, test_size = 2)

        main(X_train, X_test, Y_train, Y_test)
        output = get_out()
        expected = main_test(X_train, X_test, Y_train, Y_test)
        self.assertEqual(output, expected)

    def test_answer_seven(self):
        X, Y = make_blobs(n_samples=16, n_features=2, centers=2, center_box=(-2,2))
        X = MinMaxScaler().fit(X).transform(X)
        X_train, X_test, Y_train, Y_test = train_test_split(X, Y, test_size = 2)

        main(X_train, X_test, Y_train, Y_test)
        output = get_out()
        expected = main_test(X_train, X_test, Y_train, Y_test)
        self.assertEqual(output, expected)

    def test_answer_eight(self):
        X, Y = make_blobs(n_samples=16, n_features=2, centers=2, center_box=(-2,2))
        X = MinMaxScaler().fit(X).transform(X)
        X_train, X_test, Y_train, Y_test = train_test_split(X, Y, test_size = 2)

        main(X_train, X_test, Y_train, Y_test)
        output = get_out()
        expected = main_test(X_train, X_test, Y_train, Y_test)
        self.assertEqual(output, expected)

    def test_answer_nine(self):
        X, Y = make_blobs(n_samples=16, n_features=2, centers=2, center_box=(-2,2))
        X = MinMaxScaler().fit(X).transform(X)
        X_train, X_test, Y_train, Y_test = train_test_split(X, Y, test_size = 2)

        main(X_train, X_test, Y_train, Y_test)
        output = get_out()
        expected = main_test(X_train, X_test, Y_train, Y_test)
        self.assertEqual(output, expected)

    def test_answer_ten(self):
        X, Y = make_blobs(n_samples=16, n_features=2, centers=2, center_box=(-2,2))
        X = MinMaxScaler().fit(X).transform(X)
        X_train, X_test, Y_train, Y_test = train_test_split(X, Y, test_size = 2)

        main(X_train, X_test, Y_train, Y_test)
        output = get_out()
        expected = main_test(X_train, X_test, Y_train, Y_test)
        self.assertEqual(output, expected)


if __name__ == '__main__':
    unittest.main()
